'use strict';

/**
this class function getServiceResource() output the object
{
    "Service": "",
    "Stage": "",
    "ServiceFileName": "fileName.json",
    "ServiceZipName": "fileName.zip",
    "CreateTime": "2019-10-17T19:46:22.473Z",
    "CreateTimestamp": 1571341582,
    "Resources": {
        "default": {
            "Type": "TencentCloud::Serverless::Namespace",
            "my-6function": {
                "Type": "TencentCloud::Serverless::Function",
                "Properties": {
                    "CodeUri": {
                        "Bucket": "my-deployment-bucket",
                        "Key": "fileName.zip"
                    },
                    "Type": "Event",
                    "Description": "the Description",
                    "Role": "QCS_SCFExcuteRole",
                    "Handler": "handler.hello",
                    "MemorySize": 128,
                    "Timeout": 20,
                    "Region": "us-east-1",
                    "Runtime": "Nodejs8.10",
                    "Tags": {
                        "key1": "val1",
                        "key2": "val2",
                    },
                    "VpcConfig": {
                        "VpcId": "",
                        "SubnetId": ""
                    },
                    "Environment": {
                        "Variables": {
                            "key1": "val1",
                            "key2": "val2",
                        }
                    }
                }
            }
        }
    }
}
*/

class Provider {

    constructor(inputs, prefix) {
        this.inputs = inputs;
        this.prefix = prefix || 'sls-cloudfunction';
    }

    // 
    getFunctionName(functionName) {
        return functionName;
    }

    randomString() {
        const len = 6;
        const chars = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678';
        const maxPos = chars.length;
        let result = '';
        for (let i = 0; i < len; i++) {
            result += chars.charAt(Math.floor(Math.random() * maxPos));
        }
        return result;
    }

    get region() {
        return this.inputs.region || 'ap-guangzhou';
    }

    getFuntionBucketKey(ns, functionName) {
        const nowDate = new Date();
        const timestamp = parseInt(nowDate.getTime() / 1000);
        return this.prefix + '-' + ns + '-' + functionName + '-' + timestamp + '.zip';
    }

    getServiceFileName(ns, funcName, timeData) {
        return this.prefix + '-' + ns + '-' + funcName + '-' + timeData;
    }

    getEnvironment(funcObject) {
        const funcObjectInfo = funcObject.environment;
        if (funcObjectInfo && funcObjectInfo.variables) 
            return {Variables: funcObject.environment.variables};
        else 
            return null;
    }

    getVPCConfig(funcObject) {
        const funcObjectInfo = funcObject.vpcConfig;
        const vpcId = (funcObjectInfo && funcObjectInfo.vpcId) ? funcObjectInfo.vpcId : '';
        const subnetId = (funcObjectInfo && funcObjectInfo.subnetId) ? funcObjectInfo.subnetId : '';
        return {'VpcId': vpcId, 'SubnetId': subnetId}
    }

    getCosBucketNme(name) {
        return this.prefix + '-' + this.region + '-' + (name ? name : 'code');
    }


    getFunctionResource(funcObject, functionName, serviceStr, keyTime) {
        const functionResource = {
            "Type": "TencentCloud::Serverless::Function",
            'Properties': {
                'CodeUri': {
                    'Bucket': this.getCosBucketNme(),
                    'Key': this.getFuntionBucketKey(this.namespace, functionName)
                },
                "Type": "Event",
                'Description': funcObject.description || "This is a template function",
                'Role': funcObject.role || "QCS_SCFExcuteRole",
                'Handler': funcObject.handler || "index.main_handler",
                'MemorySize': funcObject.memorySize || 128,
                'Timeout': funcObject.timeout || 3,
                'Region': funcObject.region || "ap-guangzhou",
                'Runtime': funcObject.runtime || 'Nodejs8.9',
                'Tags': {
                    "CLI": "Serverless",
                    "Application": 'component',
                    "Stage": this.stage
                }
            }
        };
  
        const vpcConfig = this.getVPCConfig(funcObject);
        if (vpcConfig)
            functionResource['Properties']['VpcConfig'] = vpcConfig

        const environment = this.getEnvironment(funcObject);
        if (environment)
            functionResource['Properties']['Environment'] = environment;

        return functionResource
    }

/*
    getTimerEvent(event, funcObject) {
        const trigger = {};
        trigger[event.name] = {
            'Type': 'Timer',
            'Properties': {
                'CronExpression': event.parameters.cronExpression,
                'Enable': event.parameters.enable
            }
        };
        return trigger;
    }

    getCOSEvent(event, funcObject) {
        const trigger = {};
        const filter = {};
        const prefix = event.parameters.filter && event.parameters.filter.prefix ? event.parameters.filter.prefix : null;
        const suffix = event.parameters.filter && event.parameters.filter.suffix ? event.parameters.filter.suffix : null;
        if (prefix) {
            filter['Prefix'] = prefix
        }
        if (suffix) {
            filter['Suffix'] = suffix
        }
        trigger[event.name] = {
            'Type': 'COS',
            'Properties': {
                'Bucket': event.parameters.bucket,
                'Events': event.parameters.events,
                'Enable': event.parameters.enable
            }
        };
        if (this.jsonObjectIsEmpty(filter) == false) {
            trigger[event.name]['Properties']['Filter'] = filter
        }
        return trigger;
    }

    getAPIGWEvent(event, funcObject) {
        const trigger = {};
        const apiGateway = this.serverless.service.provider;
        trigger[event.name] = {
            'Type': 'APIGW',
            'Properties': {
                'StageName': event.parameters.stageName,
                'HttpMethod': event.parameters.httpMethod,
                'ServiceId': event.parameters.serviceId || (apiGateway && apiGateway.serviceId ? apiGateway.serviceId : ""),
                'IntegratedResponse': event.parameters.integratedResponse,
                'Enable': event.parameters.enable
            }
        };
        return trigger;
    }

    getCMQEvent(event, funcObject) {
        const trigger = {};
        trigger[event.name] = {
            'Type': 'CMQ',
            'Properties': {
                'Name': event.parameters.name,
                'Enable': event.parameters.enable
            }
        };
        return trigger;
    }

    getCkafkaEvent(event, funcObject) {
        const trigger = {};
        trigger[event.name] = {
            'Type': 'Ckafka',
            'Properties': {
                'Name': event.parameters.name,
                'Topic': event.parameters.topic,
                'MaxMsgNum': event.parameters.maxMsgNum,
                'Offset': event.parameters.offset,
                'Enable': event.parameters.enable
            }
        };
        return trigger;
    }

    getDeployCosBucket(short) {
        if (short) {
            const cousBucket = this.serverless.service.provider.cosBucket || "DEFAULT";
            return cousBucket != "DEFAULT" ? cousBucket : 'sls-cloudfunction-' + this.getRegion();
        } else {
            const cousBucket = this.serverless.service.provider.cosBucket || "DEFAULT";
            return cousBucket != "DEFAULT" ? cousBucket : 'sls-cloudfunction-' + this.getRegion() + "-" + this.options.credentials.tencent_appid;
        }
    }
*/
    get namespace() {
        return 'default';
    }

    get service() {
        return '';
    }

    get stage() {
        return '';
    }

    getServiceResource() {
        var functionList = {
            "Type": "TencentCloud::Serverless::Namespace",
        };
        const serviceStr = this.randomString();
        const date = new Date();
        date.setMinutes(date.getMinutes() - date.getTimezoneOffset()); // toJSON 的时区补偿
        const keyTime = date.toJSON().substr(0, 19).replace(/[:T]/g, '-');
        const funcObject = this.inputs;
        const functionName = this.getFunctionName(funcObject.name);
        const funtionResource = this.getFunctionResource(funcObject, functionName, serviceStr, keyTime);
        functionList[functionName] = funtionResource

        const resource = {
            "Service": this.service,
            "Stage": this.stage,
            "ServiceFileName": this.getServiceFileName(this.namespace, functionName, Date.parse(date) / 1000) + ".json",
            "ServiceZipName": this.getFuntionBucketKey(this.namespace, functionName),
            "CreateTime": date,
            "CreateTimestamp": Date.parse(date) / 1000,
            "Resources": {}
        };

        resource["Resources"]["default"] = functionList;
        return resource
    }

}

module.exports = Provider;
